/**
 * Ein Knoten einer allgemeinen Liste.
 * 
 * @author Bernd Gramlich 
 * @version 2015-12-02
 */
public class Knoten extends Listenelement
{
    /*
     * Inhalt des Knotens ist ein Eintrag.
     */
    private Eintrag inhalt;

    /*
     * Nachfolger des Knotens ist ein Listenelement,
     * also entweder wieder ein Knoten oder der Abschluss
     * der Liste.
     */
    private Listenelement nachfolger;

    /**
     * Konstruktor.
     * Erzeuge einen neuen Knoten mit Inhalt "meinInhalt" und
     * Nachfolger "meinNachfolger".
     */
    public Knoten (Listenelement meinNachfolger, Eintrag meinInhalt)
    {
        nachfolger = meinNachfolger;
        inhalt     = meinInhalt;
    }

    /**
     * Nenne den Nachfolger des Knotens.
     */
    public Listenelement gibNachfolger ()
    {
        return nachfolger;
    }

    /**
     * Nenne den Inhalt des Knotens.
     */
    public Eintrag gibInhalt ()
    {
        return inhalt;
    }

    /**
     * Verschaffe dem Knoten einen neuen Nachfolger.
     */
    public void setzeNachfolger (Listenelement neuerNachfolger)
    {
        nachfolger = neuerNachfolger;
    }

    /**
     * Fge hinten an die Liste ein neues Listenelement an.
     */
    public Listenelement fuegeHintenAn (Eintrag neuerEintrag)
    {
        /*
         * Ich berlasse meinem Nachfolger die Aufgabe, den neuen
         * Kunden hinten anzustellen. Wenn mein Nachfolger der 
         * Abschluss ist, dann tut er das auch tatschlich. Wenn
         * mein Nachfolger ein weiterer Knoten ist, dann wird er
         * den Befehl innerhalb der Schlange weiterreichen.
         * 
         * Graphisch muss ich den neuen Eintrag zuvor nach rechts
         * verschieben.
         */
        neuerEintrag.verschiebeNachRechts ();

        nachfolger = nachfolger.fuegeHintenAn (neuerEintrag);

        return this;
    }

    /**
     * Fge einen neuen Eintrag aufsteigend sortiert in die Liste ein.
     * 
     * Da ich der Abschluss bin, erzeuge ich einen neuen
     * Knoten mit mir als Nachfolger. Den Verweis auf mich
     * selbst erziele ich durch das Java-Schlsselwort "this".
     */
    public Listenelement fuegePassendEin (Eintrag neuerEintrag)
    {
        if (inhalt.istKleinerAls (neuerEintrag))
        {
            /*
             * Wenn mein Inhalt kleiner ist als der neue Eintrag, dann
             * muss der neue Eintrag hinter mir eingefgt werden. Diese
             * Aufgabe berlasse ich meinem Nachfolger.
             * 
             * Graphisch muss ich den neuen Eintrag zuvor nach rechts
             * verschieben.
             */
            neuerEintrag.verschiebeNachRechts ();
            
            nachfolger = nachfolger.fuegePassendEin (neuerEintrag);

            return this;
        }
        else
        {
            /*
             * Wenn der neue Eintrag hchstens so gro ist wie mein
             * Inhalt, dann fge ihn vor mir ein.
             * 
             * Graphisch muss ich zuvor die Liste ab mir nach rechts
             * verschieben.
             */
            verschiebeNachRechts ();
            
            return new Knoten (this, neuerEintrag);
        }
    }

    /**
     * Nenne die Lnge der Schlange von mir selbst ab.
     */
    public int gibLaenge ()
    {
        return 1 + nachfolger.gibLaenge ();
    }

    /**
     * Verschiebe den Knoten und alle seine Nachfolger
     * graphisch nach links.
     */
    public void verschiebeNachLinks ()
    {
        /*
         * Ich verschiebe meinen Inhalt graphisch nach
         * links.
         */
        inhalt.verschiebeNachLinks ();

        /*
         * Ich fordere meinen Nachfolger auf, sich ebenfalls nach
         * links zu verschieben.
         */
        nachfolger.verschiebeNachLinks ();
    }

    /**
     * Verschiebe den Knoten und alle seine Nachfolger 
     * graphisch nach rechts.
     */
    public void verschiebeNachRechts ()
    {
        /*
         * Ich fordere meinen Nachfolger auf, sich ebenfalls nach
         * rechts zu verschieben.
         */
        nachfolger.verschiebeNachRechts ();

        /*
         * Ich verschiebe meinen Inhalt graphisch nach
         * rechts.
         */
        inhalt.verschiebeNachRechts ();
    }
    
    /**
     * Drucke den Inhalt der Liste ab mir selbst auf die
     * Konsole.
     */
    public void druckeAufKonsole ()
    {
        inhalt.druckeAufKonsole ();
        nachfolger.druckeAufKonsole ();
    }
}
