/**
 * Ein symbolisches Tretauto.
 * 
 * @author Bernd Gramlich
 * @version 2015-12-02
 */
public class Tretauto
{
    // Bestandteile als Referenzen auf Graphikobjekte.
    private Vieleck karosserie;
    private Ellipse vorderrad;
    private Ellipse hinterrad;
    
    /**
     * Konstruktor.
     * Erzeuge ein Auto der Strichstrke a
     * an der Position (x|y) mit den gewnschten Farben.
     */
    public Tretauto (int a, int x, int y,
                      String karosseriefarbe, String reifenfarbe)
    {
        karosserie = new Vieleck ();
        vorderrad  = new Ellipse ();
        hinterrad  = new Ellipse ();
        
        karosserie.ergaenzeEcke (x       , y + 65*a);
        karosserie.ergaenzeEcke (x       , y + 76*a);
        karosserie.ergaenzeEcke (x + 33*a, y + 76*a);
        karosserie.ergaenzeEcke (x + 33*a, y + 65*a);
        karosserie.ergaenzeEcke (x + 28*a, y + 65*a);
        karosserie.ergaenzeEcke (x + 18*a, y + 55*a);
        karosserie.ergaenzeEcke (x +  8*a, y + 55*a);
        karosserie.ergaenzeEcke (x +  3*a, y + 65*a);
        karosserie.setzeFarbe (karosseriefarbe);
        
        hinterrad.setzeBreite (8*a);
        hinterrad.setzeHoehe  (8*a);
        hinterrad.setzePosition (x +  3*a, y + 72*a);
        hinterrad.setzeFarbe (reifenfarbe);
        
        vorderrad.setzeBreite (8*a);
        vorderrad.setzeHoehe  (8*a);
        vorderrad.setzePosition (x + 22*a, y + 72*a);
        vorderrad.setzeFarbe (reifenfarbe);
        
        werdeSichtbar ();
    }
    
    /**
     * Mache das Auto sichtbar.
     */
    public void werdeSichtbar ()
    {
        karosserie.werdeSichtbar ();        
        vorderrad.werdeSichtbar ();
        hinterrad.werdeSichtbar ();
    }
    
    /**
     * Mache das Auto unsichtbar.
     */
    public void werdeUnsichtbar ()
    {
        karosserie.werdeUnsichtbar ();        
        vorderrad.werdeUnsichtbar ();
        hinterrad.werdeUnsichtbar ();
    }
    
    /**
     * Verschiebe das Auto um den Vektor (dx|dy).
     */
    public void verschiebe (int dx, int dy)
    {
        werdeUnsichtbar ();
        
        karosserie.verschiebe (dx, dy);        
        vorderrad.verschiebeWaagrecht (dx);
        vorderrad.verschiebeSenkrecht (dy);
        hinterrad.verschiebeWaagrecht (dx);
        hinterrad.verschiebeSenkrecht (dy);
        
        werdeSichtbar ();
    }
    
    /**
     * Nenne die Karosseriefarbe.
     */
    public String gibKarosseriefarbe ()
    {
        return karosserie.gibFarbe ();
    }
    
    /**
     * Nenne die Reifenfarbe.
     */
    public String gibReifenfarbe ()
    {
        return vorderrad.gibFarbe ();
    }

    /**
     * Erzeuge ein zuflliges Tretauto der Strichstrke 3
     * und zeichne sie in der linken oberen Leinwandecke.
     */
    public static Tretauto erzeugeZufaellig ()
    {
        String karosseriefarbe;
        String reifenfarbe;
        
        int zufall;

        zufall = (int) Math.floor (10 * Math.random ());
            
        switch (zufall)
        {
            case 0:
            case 1:  karosseriefarbe = "rot"; break;
            case 2:
            case 3:  karosseriefarbe = "pink"; break;
            case 4:
            case 5:  karosseriefarbe = "lila"; break;
            case 6:  karosseriefarbe = "gelb"; break;
            case 7:  karosseriefarbe = "orange"; break;
            case 8:  karosseriefarbe = "trkis"; break;
            default: karosseriefarbe = "braun"; break;
        }

        zufall = (int) Math.floor (5 * Math.random ());

        switch (zufall)
        {
            case 0:  reifenfarbe = "grau"; break;
            default: reifenfarbe = "schwarz"; break;
        }
            
        return new Tretauto (3, 0, 0, karosseriefarbe, reifenfarbe);
    }
}